Stan Math Library: stan/math/prim/scal/prob/poisson_cdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

poisson_cdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_POISSON_CDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_POISSON_CDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_less.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/multiply_log.hpp>
13 #include <stan/math/prim/scal/fun/gamma_q.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <boost/math/special_functions/fpclassify.hpp>
16 #include <boost/random/poisson_distribution.hpp>
17 #include <boost/random/variate_generator.hpp>
18 #include <cmath>
19 #include <limits>
20
21 namespace stan {
22
23 namespace math {
24
25 template <typename T_n, typename T_rate>
26 typename return_type<T_rate>::type
27 poisson_cdf_log(const T_n& n, const T_rate& lambda) {
28 static const char* function("stan::math::poisson_cdf_log");
29 typedef typename stan::partials_return_type<T_n, T_rate>::type
30 T_partials_return;
31
32 using stan::math::check_not_nan;
33 using stan::math::check_nonnegative;
34 using stan::math::value_of;
35 using stan::math::check_consistent_sizes;
36
37 // Ensure non-zero argument slengths
38 if (!(stan::length(n) && stan::length(lambda)))
39 return 0.0;
40
41 T_partials_return P(0.0);
42
43 // Validate arguments
44 check_not_nan(function, "Rate parameter", lambda);
45 check_nonnegative(function, "Rate parameter", lambda);
46 check_consistent_sizes(function,
47 "Random variable", n,
48 "Rate parameter", lambda);
49
50 // Wrap arguments into vector views
51 VectorView<const T_n> n_vec(n);
52 VectorView<const T_rate> lambda_vec(lambda);
53 size_t size = max_size(n, lambda);
54
55 // Compute vectorized cdf_log and gradient
56 using stan::math::value_of;
57 using stan::math::gamma_q;
58 using boost::math::tgamma;
59 using std::exp;
60 using std::pow;
61 using std::log;
62 using std::exp;
63
64 OperandsAndPartials<T_rate> operands_and_partials(lambda);
65
66 // Explicit return for extreme values
67 // The gradients are technically ill-defined, but treated as neg infinity
68 for (size_t i = 0; i < stan::length(n); i++) {
69 if (value_of(n_vec[i]) < 0)
70 return operands_and_partials.value(stan::math::negative_infinity());
71 }
72
73 for (size_t i = 0; i < size; i++) {
74 // Explicit results for extreme values
75 // The gradients are technically ill-defined, but treated as zero
76 if (value_of(n_vec[i]) == std::numeric_limits<int>::max())
77 continue;
78
79 const T_partials_return n_dbl = value_of(n_vec[i]);
80 const T_partials_return lambda_dbl = value_of(lambda_vec[i]);
81 const T_partials_return Pi = gamma_q(n_dbl+1, lambda_dbl);
82
83 P += log(Pi);
84
85 if (!is_constant_struct<T_rate>::value)
86 operands_and_partials.d_x1[i] -= exp(-lambda_dbl)
87 * pow(lambda_dbl, n_dbl) / tgamma(n_dbl+1) / Pi;
88 }
89
90 return operands_and_partials.value(P);
91 }
92 }
93 }
94 #endif
check_not_nan.hpp
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
gamma_q.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::poisson_cdf_log
return_type< T_rate >::type poisson_cdf_log(const T_n &n, const T_rate &lambda)
Definition: poisson_cdf_log.hpp:27
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
stan::math::max
int max(const std::vector< int > &x)
Returns the maximum coefficient in the specified column vector.
Definition: max.hpp:21
partials_return_type.hpp
OperandsAndPartials.hpp
constants.hpp
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::pow
fvar< T > pow(const fvar< T > &x1, const fvar< T > &x2)
Definition: pow.hpp:18
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::math::tgamma
fvar< T > tgamma(const fvar< T > &x)
Definition: tgamma.hpp:15
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_less.hpp
stan::math::gamma_q
fvar< T > gamma_q(const fvar< T > &x1, const fvar< T > &x2)
Definition: gamma_q.hpp:15
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::negative_infinity
double negative_infinity()
Return negative infinity.
Definition: constants.hpp:132

[ Stan Home Page ] © 2011–2016, Stan Development Team.